@@ -3,7 +3,7 @@
 #
 #  (C) Paul Evans, 2008-2023 -- leonerd@leonerd.org.uk
 
-package String::Tagged 0.20;
+package String::Tagged 0.21;
 
 use v5.14;
 use warnings;
@@ -12,8 +12,12 @@ use Scalar::Util qw( blessed );
 
 require String::Tagged::Extent;
 
-use constant FLAG_ANCHOR_BEFORE => 0x01;
-use constant FLAG_ANCHOR_AFTER  => 0x02;
+use constant {
+   FLAG_ANCHOR_BEFORE => 0x01,
+   FLAG_ANCHOR_AFTER  => 0x02,
+   FLAG_ITERATING     => 0x04,
+   FLAG_DELETED       => 0x08,
+};
 
 use constant DEBUG => 0;
 
@@ -721,6 +725,14 @@ sub _remove_tag
 
    my ( $name ) = @_;
 
+   if( my $t = $self->{iterating} ) {
+      my ( $ts, $te, $tn ) = @$t;
+      if( $start == $ts and $end == $te and $name eq $tn ) {
+         $t->[4] |= FLAG_DELETED;
+         return;
+      }
+   }
+
    my $tags = $self->{tags};
 
    my $have_added = 0;
@@ -735,14 +747,18 @@ sub _remove_tag
       next if $tn ne $name;
 
       if( $keepends and $end < $te ) {
-         $self->_insert_tag( $end, $te, $tn, $tv, $tf & ~FLAG_ANCHOR_BEFORE );
+         $self->_insert_tag( $end, $te, $tn, $tv, $tf & ~(FLAG_ANCHOR_BEFORE|FLAG_ITERATING) );
          $have_added = 1;
       }
 
+      if( $tf & FLAG_ITERATING ) {
+         die "ARGH encountered FLAG_ITERATING while walking the list of tags during ->_remove_tag";
+      }
+
       splice @$tags, $i, 1;
 
       if( $keepends and $ts < $start ) {
-         $self->_insert_tag( $ts, $start, $tn, $tv, $tf & ~FLAG_ANCHOR_AFTER );
+         $self->_insert_tag( $ts, $start, $tn, $tv, $tf & ~(FLAG_ANCHOR_AFTER|FLAG_ITERATING) );
          $have_added = 1;
       }
       else {
@@ -801,6 +817,39 @@ sub delete_tag
    return $self->_remove_tag( 0, @_ );
 }
 
+=head2 delete_all_tag
+
+   $st->delete_all_tag( $name )
+
+I<Since version 0.21.>
+
+Deletes every tag with the given name. This is more efficient than calling
+C<iter_extents> to list the tags then C<delete_tag> on each one individually
+in the case of a simple name match.
+
+This method returns the C<$st> object.
+
+=cut
+
+sub delete_all_tag
+{
+   my $self = shift;
+   my ( $name ) = @_;
+
+   my $tags = $self->{tags};
+
+   for( my $i = 0; $i < @$tags; $i++ ) {
+      my ( $ts, $te, $tn, $tv, $tf ) = @{ $tags->[$i] };
+
+      next if $tn ne $name;
+
+      splice @$tags, $i, 1, ();
+      $i--;
+   }
+
+   return $self;
+}
+
 =head2 merge_tags
 
    $st->merge_tags( $eqsub )
@@ -905,6 +954,17 @@ Select all the tags except those named in the given ARRAY reference.
 
 =back
 
+I<Since version 0.21> it is safe to call C<delete_tag> from within the
+callback function to remove the tag currently being iterated on. 
+
+   $str->iter_extents( sub {
+      my ( $e, $n, $v ) = @_;
+      $str->delete_tag( $e, $n ) if $n =~ m/^tmp_/;
+   } );
+
+Apart from this scenario, the tags in the string should not otherwise be added
+or removed while the iteration is occurring.
+
 =cut
 
 sub iter_extents
@@ -927,7 +987,8 @@ sub iter_extents
 
    my $tags = $self->{tags};
 
-   foreach my $t ( @$tags ) {
+   for ( my $i = 0; $i < @$tags; $i++ ) {
+      my $t = $tags->[$i];
       my ( $ts, $te, $tn, $tv, $tf ) = @$t;
 
       next if $te < $start;
@@ -936,7 +997,17 @@ sub iter_extents
       next if $only   and !$only->{$tn};
       next if $except and  $except->{$tn};
 
+      $t->[4] |= FLAG_ITERATING;
+      local $self->{iterating} = $t;
+
       $callback->( $self->_mkextent( $ts, $te, $tf ), $tn, $tv );
+
+      $t->[4] &= ~FLAG_ITERATING;
+
+      if( $t->[4] & FLAG_DELETED ) {
+         splice @$tags, $i, 1, ();
+         $i--;
+      }
    }
 }
 
